@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from __future__ import print_function
 
 import atexit
@@ -26,16 +40,28 @@ try:
 except ImportError:
     import builtins as __builtin__
 
+def _argspec_py2(func):
+    return inspect.getargspec(func)
+
+def _argspec_py3(func):
+    a = inspect.getfullargspec(func)
+    return (a.args, a.varargs, a.varkw, a.defaults)
+
+if hasattr(inspect, 'getfullargspec'):
+    _argspec = _argspec_py3
+else:
+    _argspec = _argspec_py2
+
 from newrelic.core.agent import agent_instance
 from newrelic.core.config import global_settings, flatten_settings
 from newrelic.api.transaction import Transaction
 from newrelic.api.object_wrapper import ObjectWrapper
-from newrelic.core.transaction_cache import transaction_cache
+from newrelic.core.trace_cache import trace_cache
 
-_transaction_cache = transaction_cache()
+_trace_cache = trace_cache()
 
 def shell_command(wrapped):
-    args, varargs, keywords, defaults = inspect.getargspec(wrapped)
+    args, varargs, keywords, defaults = _argspec(wrapped)
 
     parser = optparse.OptionParser()
     for name in args[1:]:
@@ -339,7 +365,7 @@ class ConsoleShell(cmd.Cmd):
         """
         """
 
-        for item in _transaction_cache.active_threads():
+        for item in _trace_cache.active_threads():
             transaction, thread_id, thread_type, frame = item
             print('THREAD', item, file=self.stdout)
             if transaction is not None:
@@ -581,14 +607,14 @@ class ClientShell(cmd.Cmd):
                     if self.__log_object:
                         self.__log_object.write(c)
 
-                    client.sendall(c)
+                    client.sendall(c.encode('utf-8'))
                 except Exception:
                     break
 
         def read():
             while 1:
                 try:
-                    c = client.recv(1)
+                    c = client.recv(1).decode('utf-8')
 
                     if not c:
                         break
