@@ -1,15 +1,30 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 """This module holds the Agent class which is the primary interface for
 interacting with the agent core.
 
 """
 
+from __future__ import print_function
 import os
 import sys
 import time
+import sched
 import logging
 import threading
 import atexit
-import warnings
 import traceback
 
 import newrelic
@@ -18,14 +33,15 @@ import newrelic.core.application
 
 import newrelic.packages.six as six
 
-from ..common.log_file import initialize_logging
-from ..samplers.cpu_usage import cpu_usage_data_source
-from ..samplers.memory_usage import memory_usage_data_source
+from newrelic.common.log_file import initialize_logging
+from newrelic.samplers.cpu_usage import cpu_usage_data_source
+from newrelic.samplers.memory_usage import memory_usage_data_source
 
 from newrelic.core.thread_utilization import thread_utilization_data_source
 
 _logger = logging.getLogger(__name__)
 
+
 def check_environment():
     # If running under uWSGI, then must be version 1.2.6 or newer. Must
     # also be run with '--enable-threads' option.
@@ -52,7 +68,8 @@ def check_environment():
                     'https://newrelic.com/docs/python/python-agent-and-uwsgi.',
                     '.'.join(map(str, uwsgi.version_info[:3])))
 
-        if not uwsgi.opt.get('enable-threads'):
+        if (hasattr(uwsgi, 'opt') and hasattr(uwsgi.opt, 'get') and
+                not uwsgi.opt.get('enable-threads')):
             _logger.warning('The New Relic Python Agent requires that when '
                     'using uWSGI that the enable-threads option be given '
                     'to uwsgi when it is run. If the option is not supplied '
@@ -60,6 +77,7 @@ def check_environment():
                     'data being reported by the agent. For more details see '
                     'https://newrelic.com/docs/python/python-agent-and-uwsgi.')
 
+
 class Agent(object):
 
     """Only one instance of the agent should ever exist and that can be
@@ -72,7 +90,7 @@ class Agent(object):
 
     The global configuration settings would normally be setup from the
     agent configuration file or could also be set explicitly. Direct access
-    to global configuration setings prior to the agent instance being
+    to global configuration settings prior to the agent instance being
     created needs to be via the 'newrelic.core.config' module.
 
     After the network connection details have been set, and the agent
@@ -147,39 +165,28 @@ class Agent(object):
                         'newrelic-admin command with command line of %s.',
                         os.environ['NEW_RELIC_ADMIN_COMMAND'])
 
-        instance = None
-
         with Agent._instance_lock:
             if not Agent._instance:
                 if settings.debug.log_agent_initialization:
                     _logger.info('Creating instance of Python agent in '
                             'process %d.', os.getpid())
-                    _logger.info('Agent was intialized from: %r',
+                    _logger.info('Agent was initialized from: %r',
                             ''.join(traceback.format_stack()[:-1]))
                 else:
                     _logger.debug('Creating instance of Python agent in '
                             'process %d.', os.getpid())
-                    _logger.debug('Agent was intialized from: %r',
+                    _logger.debug('Agent was initialized from: %r',
                             ''.join(traceback.format_stack()[:-1]))
 
                 instance = Agent(settings)
+                _logger.debug('Registering builtin data sources.')
+
+                instance.register_data_source(cpu_usage_data_source)
+                instance.register_data_source(memory_usage_data_source)
+                instance.register_data_source(thread_utilization_data_source)
 
                 Agent._instance = instance
 
-        if instance:
-            _logger.debug('Activating agent instance.')
-
-            instance.activate_agent()
-
-            _logger.debug('Registering builtin data sources.')
-
-            instance.register_data_source(cpu_usage_data_source)
-            instance.register_data_source(memory_usage_data_source)
-            instance.register_data_source(thread_utilization_data_source)
-
-            for callable in Agent._startup_callables:
-                callable()
-
         return Agent._instance
 
     def __init__(self, config):
@@ -202,10 +209,15 @@ class Agent(object):
         self._harvest_thread.setDaemon(True)
         self._harvest_shutdown = threading.Event()
 
-        self._harvest_count = 0
-        self._last_harvest = 0.0
-        self._harvest_duration = 0.0
-        self._next_harvest = 0.0
+        self._default_harvest_count = 0
+        self._flexible_harvest_count = 0
+        self._last_default_harvest = 0.0
+        self._last_flexible_harvest = 0.0
+        self._default_harvest_duration = 0.0
+        self._flexible_harvest_duration = 0.0
+        self._scheduler = sched.scheduler(
+                self._harvest_timer,
+                self._harvest_shutdown.wait)
 
         self._process_shutdown = False
 
@@ -240,22 +252,28 @@ class Agent(object):
     def dump(self, file):
         """Dumps details about the agent to the file object."""
 
-        print >> file, 'Time Created: %s' % (
-                time.asctime(time.localtime(self._creation_time)))
-        print >> file, 'Initialization PID: %s' % (
-                self._process_id)
-        print >> file, 'Harvest Count: %d' % (
-                self._harvest_count)
-        print >> file, 'Last Harvest: %s' % (
-                time.asctime(time.localtime(self._last_harvest)))
-        print >> file, 'Harvest Duration: %.2f' % (
-                self._harvest_duration)
-        print >> file, 'Next Harvest: %s' % (
-                time.asctime(time.localtime(self._next_harvest)))
-        print >> file, 'Agent Shutdown: %s' % (
-                self._harvest_shutdown.isSet())
-        print >> file, 'Applications: %r' % (
-                sorted(self._applications.keys()))
+        print('Time Created: %s' % (
+                time.asctime(time.localtime(self._creation_time))), file=file)
+        print('Initialization PID: %s' % (
+                self._process_id), file=file)
+        print('Default Harvest Count: %d' % (
+                self._default_harvest_count), file=file)
+        print('Flexible Harvest Count: %d' % (
+                self._flexible_harvest_count), file=file)
+        print('Last Default Harvest: %s' % (
+                time.asctime(time.localtime(self._last_default_harvest))),
+                file=file)
+        print('Last Flexible Harvest: %s' % (
+                time.asctime(time.localtime(self._last_flexible_harvest))),
+                file=file)
+        print('Default Harvest Duration: %.2f' % (
+                self._default_harvest_duration), file=file)
+        print('Flexible Harvest Duration: %.2f' % (
+                self._flexible_harvest_duration), file=file)
+        print('Agent Shutdown: %s' % (
+                self._harvest_shutdown.isSet()), file=file)
+        print('Applications: %r' % (
+                sorted(self._applications.keys())), file=file)
 
     def global_settings(self):
         """Returns the global default settings object. If access is
@@ -281,8 +299,15 @@ class Agent(object):
         if application:
             return application.configuration
 
+    def application_attribute_filter(self, app_name):
+        """Returns the attribute filter for the application."""
+
+        application = self._applications.get(app_name)
+        if application:
+            return application.attribute_filter
+
     def activate_application(self, app_name, linked_applications=[],
-                             timeout=None):
+                             timeout=None, uninstrumented_modules=None):
         """Initiates activation for the named application if this has
         not been done previously. If an attempt to trigger the
         activation of the application has already been performed,
@@ -312,13 +337,40 @@ class Agent(object):
         settings = newrelic.core.config.global_settings()
 
         if timeout is None:
-            timeout = settings.startup_timeout
+            # In serverless mode, we should always wait for startup since it
+            # should take almost no time to create the session (no network
+            # activity).
+            if settings.serverless_mode.enabled:
+                timeout = 10.0
+            else:
+                timeout = settings.startup_timeout
 
         activate_session = False
 
         with self._lock:
             application = self._applications.get(app_name, None)
             if not application:
+
+                process_id = os.getpid()
+
+                if process_id != self._process_id:
+                    _logger.warning(
+                            'Attempt to activate application in a process '
+                            'different to where the agent harvest thread was '
+                            'started. No data will be reported for this '
+                            'process with pid of %d. Creation of the harvest '
+                            'thread this application occurred in process with '
+                            'pid %d. If no data at all is being reported for '
+                            'your application, see '
+                            'https://docs.newrelic.com/docs/agents/'
+                            'python-agent/troubleshooting/'
+                            'activate-application-warning-python '
+                            'for troubleshooting steps. If the issue '
+                            'persists, please send debug logs to New Relic '
+                            'support for assistance.',
+                            process_id,
+                            self._process_id)
+
                 if settings.debug.log_agent_initialization:
                     _logger.info('Creating application instance for %r '
                             'in process %d.', app_name, os.getpid())
@@ -333,6 +385,7 @@ class Agent(object):
                 linked_applications = sorted(set(linked_applications))
                 application = newrelic.core.application.Application(
                         app_name, linked_applications)
+                application._uninstrumented = uninstrumented_modules
                 self._applications[app_name] = application
                 activate_session = True
 
@@ -355,13 +408,13 @@ class Agent(object):
 
                 application.validate_process()
 
-        # Activate the session if application was just created and wait
-        # for session activation if a timeout was specified. This may
-        # bail out early if is detected that a deadlock may occur for
-        # the period of the timeout.
+            # Activate the session if application was just created and wait
+            # for session activation if a timeout was specified. This may
+            # bail out early if is detected that a deadlock may occur for
+            # the period of the timeout.
 
-        if activate_session:
-            application.activate_session(timeout)
+            if activate_session:
+                application.activate_session(self.activate_agent, timeout)
 
     @property
     def applications(self):
@@ -380,7 +433,7 @@ class Agent(object):
     def application(self, app_name):
         """Returns the internal application object for the named
         application or None if not created. When an application object
-        is returned, it does not relect whether activation has been
+        is returned, it does not reflect whether activation has been
         successful or not. To determine if application is currently in an
         activated state use application_settings() method to see if a valid
         application settings objects is available or query the application
@@ -421,6 +474,33 @@ class Agent(object):
                     instance.register_data_source(source, name,
                             settings, **properties)
 
+    def remove_thread_utilization(self):
+
+        _logger.debug('Removing thread utilization data source from all '
+                'applications')
+
+        source_name = thread_utilization_data_source.__name__
+        factory_name = 'Thread Utilization'
+
+        with self._lock:
+            source_names = [s[0].__name__ for s in self._data_sources[None]]
+            if source_name in source_names:
+                idx = source_names.index(source_name)
+                self._data_sources[None].pop(idx)
+
+            # Clear out the data samplers that add thread utilization custom
+            # metrics every harvest (for each application)
+
+            for application in self._applications.values():
+                application.remove_data_source(factory_name)
+
+        # The thread utilization data source may have been started, so we
+        # must clear out the list of trackers that transactions will use to add
+        # thread.concurrency attributes
+
+        from newrelic.core.thread_utilization import _utilization_trackers
+        _utilization_trackers.clear()
+
     def record_exception(self, app_name, exc=None, value=None, tb=None,
             params={}, ignore_errors=[]):
 
@@ -443,13 +523,6 @@ class Agent(object):
 
         application.record_custom_metric(name, value)
 
-    def record_metric(self, app_name, name, value):
-        warnings.warn('Internal API change. Use record_custom_metric() '
-                'instead of record_metric().', DeprecationWarning,
-                stacklevel=2)
-
-        return self.record_custom_metric(app_name, name, value)
-
     def record_custom_metrics(self, app_name, metrics):
         """Records the metrics for the named application. If there has
         been no prior request to activate the application, the metric is
@@ -464,14 +537,14 @@ class Agent(object):
 
         application.record_custom_metrics(metrics)
 
-    def record_metrics(self, app_name, metrics):
-        warnings.warn('Internal API change. Use record_custom_metrics() '
-                'instead of record_metrics().', DeprecationWarning,
-                stacklevel=2)
+    def record_custom_event(self, app_name, event_type, params):
+        application = self._applications.get(app_name, None)
+        if application is None or not application.active:
+            return
 
-        return self.record_custom_metrics(app_name, metrics)
+        application.record_custom_event(event_type, params)
 
-    def record_transaction(self, app_name, data, profile_samples=None):
+    def record_transaction(self, app_name, data):
         """Processes the raw transaction data, generating and recording
         appropriate metrics against the named application. If there has
         been no prior request to activate the application, the metric is
@@ -483,7 +556,11 @@ class Agent(object):
         if application is None or not application.active:
             return
 
-        application.record_transaction(data, profile_samples)
+        application.record_transaction(data)
+
+        if self._config.serverless_mode.enabled:
+            application.harvest(flexible=True)
+            application.harvest(flexible=False)
 
     def normalize_name(self, app_name, name, rule_type='url'):
         application = self._applications.get(app_name, None)
@@ -492,65 +569,90 @@ class Agent(object):
 
         return application.normalize_name(name, rule_type)
 
+    def compute_sampled(self, app_name):
+        application = self._applications.get(app_name, None)
+        return application.compute_sampled()
+
+    def _harvest_flexible(self, shutdown=False):
+        if not self._harvest_shutdown.isSet():
+            event_harvest_config = self.global_settings().event_harvest_config
+
+            self._scheduler.enter(
+                    event_harvest_config.report_period_ms / 1000.0,
+                    1,
+                    self._harvest_flexible,
+                    ())
+            _logger.debug('Commencing harvest[flexible] of application data.')
+        elif not shutdown:
+            return
+        else:
+            _logger.debug('Commencing final harvest[flexible] of application data.')
+
+        self._flexible_harvest_count += 1
+        self._last_flexible_harvest = time.time()
+
+        for application in list(six.itervalues(self._applications)):
+            try:
+                application.harvest(shutdown=False, flexible=True)
+            except Exception:
+                _logger.exception('Failed to harvest data '
+                                  'for %s.' % application.name)
+
+        self._flexible_harvest_duration = \
+                time.time() - self._last_flexible_harvest
+
+        _logger.debug('Completed harvest[flexible] of application data in %.2f '
+                'seconds.', self._flexible_harvest_duration)
+
+    def _harvest_default(self, shutdown=False):
+        if not self._harvest_shutdown.isSet():
+            self._scheduler.enter(60.0, 2, self._harvest_default, ())
+            _logger.debug('Commencing harvest[default] of application data.')
+        elif not shutdown:
+            return
+        else:
+            _logger.debug('Commencing final harvest[default] of application data.')
+
+        self._default_harvest_count += 1
+        self._last_default_harvest = time.time()
+
+        for application in list(six.itervalues(self._applications)):
+            try:
+                application.harvest(shutdown, flexible=False)
+            except Exception:
+                _logger.exception('Failed to harvest data '
+                                  'for %s.' % application.name)
+
+        self._default_harvest_duration = \
+                time.time() - self._last_default_harvest
+
+        _logger.debug('Completed harvest[default] of application data in %.2f '
+                'seconds.', self._default_harvest_duration)
+
+    def _harvest_timer(self):
+        if self._harvest_shutdown.isSet():
+            return float("inf")
+        return time.time()
+
     def _harvest_loop(self):
         _logger.debug('Entering harvest loop.')
 
         settings = newrelic.core.config.global_settings()
+        event_harvest_config = settings.event_harvest_config
 
-        self._next_harvest = time.time()
+        self._scheduler.enter(
+                event_harvest_config.report_period_ms / 1000.0,
+                1,
+                self._harvest_flexible,
+                ())
+        self._scheduler.enter(
+                60.0,
+                2,
+                self._harvest_default,
+                ())
 
         try:
-            while True:
-                if self._harvest_shutdown.isSet():
-                    # NOTE We would have just finished a harvest or only
-                    # just started the agent, so we could skip doing a
-                    # forced harvest, or at least if most recent harvest
-                    # was started within in certain period of time. The
-                    # chances of it occuring are probably slim enough
-                    # that is not an issue.
-
-                    self._run_harvest(shutdown=True)
-
-                    return
-
-                # We are either going into the loop the first time, or
-                # something really went wrong here and we are overdue
-                # already for next harvest. This can happen when we have
-                # a large number of applications. Can also happen if
-                # clock is changed significantly. Skip it and wait until
-                # the next harvest time instead.
-                #
-                # NOTE This does mean that we aren't going to report on
-                # 1 minute intervals when have lots of applications. We
-                # need to look at using multiple threads when have lots
-                # of applications. Also need to fix problem whereby one
-                # all applications created, that only the first
-                # application will reliably report on an even minute as
-                # when the others report will depend on how long the
-                # first takes.
-
-                now = time.time()
-                while self._next_harvest <= now:
-                    self._next_harvest += 60.0
-
-                # Wait until next harvest period but drop out and force
-                # harvest if been notified that process is being
-                # shutdown.
-
-                delay = self._next_harvest - now
-                self._harvest_shutdown.wait(delay)
-
-                if self._harvest_shutdown.isSet():
-                    # Force a final harvest on agent shutdown.
-
-                    self._run_harvest(shutdown=True)
-
-                    return
-
-                # Run the normal harvest cycle.
-
-                self._run_harvest(shutdown=False)
-
+            self._scheduler.run()
         except Exception:
             # An unexpected error, possibly some sort of internal agent
             # implementation issue or more likely due to modules being
@@ -572,52 +674,36 @@ class Agent(object):
                         'loop. Please report this problem to New Relic '
                         'support for further investigation.')
 
-    def _run_harvest(self, shutdown=False):
-        # This isn't going to maintain order of applications
-        # such that oldest is always done first. A new one could
-        # come in earlier once added and upset the overall
-        # timing. The data collector should cope with this
-        # though.
-
-        if shutdown:
-            _logger.debug('Commencing harvest of all application data and '
-                    'forcing a shutdown at the same time.')
-        else:
-            _logger.debug('Commencing harvest of all application data.')
-
-        self._harvest_count += 1
-        self._last_harvest = time.time()
-
-        for application in list(six.itervalues(self._applications)):
-              try:
-                  application.harvest(shutdown)
-
-              except Exception:
-                  _logger.exception('Failed to harvest data '
-                                    'for %s.' % application.name)
-
-        self._harvest_duration = time.time() - self._last_harvest
-
-        _logger.debug('Completed harvest of all application data in %.2f '
-                'seconds.', self._harvest_duration)
-
     def activate_agent(self):
         """Starts the main background for the agent."""
+        with Agent._instance_lock:
+            # Skip this if agent is not actually enabled.
+            if not self._config.enabled:
+                _logger.warning('The Python Agent is not enabled.')
+                return
+            elif self._config.serverless_mode.enabled:
+                _logger.debug(
+                        'Harvest thread is disabled due to serverless mode.')
+                return
+            elif self._config.debug.disable_harvest_until_shutdown:
+                _logger.debug('Harvest thread is disabled.')
+                return
 
-        # Skip this if agent is not actually enabled.
+            # Skip this if background thread already running.
 
-        if not self._config.enabled:
-            _logger.warning('The Python Agent is not enabled.')
-            return
+            if self._harvest_thread.isAlive():
+                return
 
-        # Skip this if background thread already running.
+            _logger.debug('Activating agent instance.')
 
-        if self._harvest_thread.isAlive():
-            return
+            for callable in self._startup_callables:
+                callable()
 
-        _logger.debug('Start Python Agent main thread.')
+            _logger.debug('Start Python Agent main thread.')
 
-        self._harvest_thread.start()
+            self._harvest_thread.start()
+
+            self._process_id = os.getpid()
 
     def _atexit_shutdown(self):
         """Triggers agent shutdown but flags first that this is being
@@ -637,8 +723,29 @@ class Agent(object):
 
         _logger.info('New Relic Python Agent Shutdown')
 
+        # Schedule final harvests. This is OK to schedule across threads since
+        # the entries will only be added to the end of the list and won't be
+        # popped until harvest_shutdown is set.
+        self._scheduler.enter(
+                float('inf'),
+                3,
+                self._harvest_flexible,
+                (True,))
+        self._scheduler.enter(
+                float('inf'),
+                4,
+                self._harvest_default,
+                (True,))
+
         self._harvest_shutdown.set()
-        self._harvest_thread.join(timeout)
+
+        if self._config.debug.disable_harvest_until_shutdown:
+            _logger.debug('Start Python Agent main thread on shutdown.')
+            self._harvest_thread.start()
+
+        if self._harvest_thread.is_alive():
+            self._harvest_thread.join(timeout)
+
 
 def agent_instance():
     """Returns the agent object. This function should always be used and
@@ -653,19 +760,28 @@ def agent_instance():
 
     return Agent.agent_singleton()
 
-def agent():
-   warnings.warn('Internal API change. Use agent_instance() '
-           'instead of agent().', DeprecationWarning, stacklevel=2)
-
-   return agent_instance()
 
 def shutdown_agent(timeout=None):
     agent = agent_instance()
     agent.shutdown_agent(timeout)
 
+
 def register_data_source(source, application=None, name=None,
         settings=None, **properties):
     agent = agent_instance()
     agent.register_data_source(source,
             application and application.name or None, name, settings,
             **properties)
+
+
+def _remove_thread_utilization():
+    agent = agent_instance()
+    agent.remove_thread_utilization()
+
+
+def remove_thread_utilization():
+    with Agent._instance_lock:
+        if Agent._instance:
+            _remove_thread_utilization()
+        else:
+            Agent.run_on_startup(_remove_thread_utilization)
