@@ -3,7 +3,7 @@ import os
 
 from newrelic import version
 from newrelic.common import system_info
-from newrelic.common.agent_http import HttpClient, ServerlessModeClient
+from newrelic.common.agent_http import ApplicationModeClient, ServerlessModeClient
 from newrelic.core.internal_metrics import internal_count_metric
 from newrelic.common.encoding_utils import (
     json_decode,
@@ -27,6 +27,7 @@ from newrelic.network.exceptions import (
     DiscardDataForRequest,
     ForceAgentDisconnect,
     ForceAgentRestart,
+    NetworkInterfaceException,
     RetryDataForRequest,
 )
 
@@ -138,7 +139,7 @@ class AgentProtocol(object):
         "VERBOSE": _logger.debug,
     }
 
-    def __init__(self, settings, host=None, client_cls=HttpClient):
+    def __init__(self, settings, host=None, client_cls=ApplicationModeClient):
         if settings.audit_log_file:
             audit_log_fp = open(settings.audit_log_file, "a")
         else:
@@ -202,9 +203,15 @@ class AgentProtocol(object):
 
     def send(self, method, payload=()):
         params, headers, payload = self._to_http(method, payload)
-        response = self.client.send_request(
-            params=params, headers=headers, payload=payload
-        )
+
+        try:
+            response = self.client.send_request(
+                params=params, headers=headers, payload=payload
+            )
+        except NetworkInterfaceException:
+            # All HTTP errors are currently retried
+            raise RetryDataForRequest
+
         status, data = response
 
         if not 200 <= status < 300:
@@ -417,7 +424,12 @@ class AgentProtocol(object):
 
     @classmethod
     def connect(
-        cls, app_name, linked_applications, environment, settings, client_cls=HttpClient
+        cls,
+        app_name,
+        linked_applications,
+        environment,
+        settings,
+        client_cls=ApplicationModeClient,
     ):
         with cls(settings, client_cls=client_cls) as preconnect:
             redirect_host = preconnect.send("preconnect")["redirect_host"]
