@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 try:
     import urlparse
 except ImportError:
@@ -5,25 +19,72 @@ except ImportError:
 
 from collections import namedtuple
 
+import newrelic.core.attribute as attribute
 import newrelic.core.trace_node
 
+from newrelic.core.node_mixin import GenericNodeMixin
 from newrelic.core.metric import TimeMetric
 
 _ExternalNode = namedtuple('_ExternalNode',
         ['library', 'url', 'method', 'children', 'start_time', 'end_time',
-        'duration', 'exclusive', 'params'])
+        'duration', 'exclusive', 'params', 'guid',
+        'agent_attributes', 'user_attributes'])
 
-class ExternalNode(_ExternalNode):
+
+class ExternalNode(_ExternalNode, GenericNodeMixin):
 
     @property
     def details(self):
         if hasattr(self, '_details'):
             return self._details
 
-        self._details = urlparse.urlparse(self.url)
+        try:
+            self._details = urlparse.urlparse(self.url or '')
+        except Exception:
+            self._details = urlparse.urlparse('http://unknown.url')
 
         return self._details
 
+    @property
+    def name(self):
+        return 'External/%s/%s/%s' % (
+                self.netloc, self.library, self.method or '')
+
+    @property
+    def url_with_path(self):
+        details = self.details
+        url = urlparse.urlunsplit((details.scheme, details.netloc,
+                details.path, '', ''))
+        return url
+
+    @property
+    def http_url(self):
+        if hasattr(self, '_http_url'):
+            return self._http_url
+
+        _, url_attr = attribute.process_user_attribute(
+                'http.url', self.url_with_path)
+
+        self._http_url = url_attr
+        return url_attr
+
+    @property
+    def netloc(self):
+        hostname = self.details.hostname or 'unknown'
+
+        try:
+            scheme = self.details.scheme.lower()
+            port = self.details.port
+        except Exception:
+            scheme = None
+            port = None
+
+        if (scheme, port) in (('http', 80), ('https', 443)):
+            port = None
+
+        netloc = port and ('%s:%s' % (hostname, port)) or hostname
+        return netloc
+
     def time_metrics(self, stats, root, parent):
         """Return a generator yielding the timed metrics for this
         external node as well as all the child nodes.
@@ -40,19 +101,7 @@ class ExternalNode(_ExternalNode):
             yield TimeMetric(name='External/allOther', scope='',
                     duration=self.duration, exclusive=self.exclusive)
 
-        hostname = self.details.hostname or 'unknown'
-
-        try:
-            scheme = self.details.scheme.lower()
-            port = self.details.port
-        except Exception:
-            scheme = None
-            port = None
-
-        if (scheme, port) in (('http', 80), ('https', 443)):
-            port = None
-
-        netloc = port and ('%s:%s' % (hostname, port)) or hostname
+        netloc = self.netloc
 
         try:
 
@@ -98,19 +147,7 @@ class ExternalNode(_ExternalNode):
 
     def trace_node(self, stats, root, connections):
 
-        hostname = self.details.hostname or 'unknown'
-
-        try:
-            scheme = self.details.scheme.lower()
-            port = self.details.port
-        except Exception:
-            scheme = None
-            port = None
-
-        if (scheme, port) in (('http', 80), ('https', 443)):
-            port = None
-
-        netloc = port and ('%s:%s' % (hostname, port)) or hostname
+        netloc = self.netloc
 
         method = self.method or ''
 
@@ -130,14 +167,28 @@ class ExternalNode(_ExternalNode):
 
         root.trace_node_count += 1
 
-        params = self.params
+        # Agent attributes
+        self.agent_attributes['http.url'] = self.http_url
 
-        details = self.details
-        url = urlparse.urlunsplit((details.scheme, details.netloc,
-                details.path, '', ''))
-
-        params['url'] = url
+        params = self.get_trace_segment_params(
+                root.settings, params=self.params)
 
         return newrelic.core.trace_node.TraceNode(start_time=start_time,
                 end_time=end_time, name=name, params=params, children=children,
                 label=None)
+
+    def span_event(self, *args, **kwargs):
+        self.agent_attributes['http.url'] = self.http_url
+        attrs = super(ExternalNode, self).span_event(*args, **kwargs)
+        i_attrs = attrs[0]
+
+        i_attrs['category'] = 'http'
+        i_attrs['span.kind'] = 'client'
+        _, i_attrs['component'] = attribute.process_user_attribute(
+                'component', self.library)
+
+        if self.method:
+            _, i_attrs['http.method'] = attribute.process_user_attribute(
+                'http.method', self.method)
+
+        return attrs
