@@ -1,14 +1,36 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import sys
 
 from newrelic.api.function_trace import FunctionTrace
 from newrelic.api.object_wrapper import ObjectWrapper, callable_name
 from newrelic.api.transaction import current_transaction
-from newrelic.api.pre_function import wrap_pre_function
+from newrelic.api.time_trace import record_exception
+from newrelic.common.object_wrapper import wrap_function_wrapper
+from newrelic.core.config import ignore_status_code
+
+
+def _nr_wrap_handle_exception(wrapped, instance, args, kwargs):
+
+    response = wrapped(*args, **kwargs)
+
+    if not ignore_status_code(response.status_code):
+        record_exception()
+
+    return response
 
-def wrap_handle_exception(*args):
-    transaction = current_transaction()
-    if transaction:
-        transaction.record_exception(*sys.exc_info())
 
 def outer_fn_wrapper(outer_fn, instance, args, kwargs):
     view_name = args[0]
@@ -44,23 +66,23 @@ def outer_fn_wrapper(outer_fn, instance, args, kwargs):
 
         transaction.set_transaction_name(name, group, priority=5)
 
-        with FunctionTrace(transaction, name, group):
-            try:
-                return inner_fn(*args, **kwargs)
-            except:  # Catch all
-                transaction.record_exception(*sys.exc_info())
-                raise
+        with FunctionTrace(name, group):
+            # django's exception handling will record errors here
+            return inner_fn(*args, **kwargs)
 
     result = outer_fn(*args, **kwargs)
 
     return ObjectWrapper(result, None, inner_fn_wrapper)
 
+
 def instrument_tastypie_resources(module):
     _wrap_view = module.Resource.wrap_view
     module.Resource.wrap_view = ObjectWrapper(
             _wrap_view, None, outer_fn_wrapper)
 
-    wrap_pre_function(module, 'Resource._handle_500', wrap_handle_exception)
+    wrap_function_wrapper(module, 'Resource._handle_500',
+            _nr_wrap_handle_exception)
+
 
 def instrument_tastypie_api(module):
     _wrap_view = module.Api.wrap_view
