@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import sys
 import threading
 import logging
@@ -5,57 +19,60 @@ import functools
 
 from newrelic.packages import six
 
-from newrelic.agent import (FunctionWrapper, callable_name,
-    wrap_error_trace, FunctionTrace, wrap_function_trace, wrap_in_function,
-    wrap_transaction_name, wrap_post_function, current_transaction,
-    WSGIApplicationWrapper, ignore_status_code, insert_html_snippet,
-    verify_body_exists, BackgroundTask, register_application,
-    wrap_function_wrapper, FunctionTraceWrapper, function_wrapper,
-    extra_settings, global_settings)
+from newrelic.api.application import register_application
+from newrelic.api.background_task import BackgroundTask
+from newrelic.api.error_trace import wrap_error_trace
+from newrelic.api.function_trace import (FunctionTrace, wrap_function_trace,
+        FunctionTraceWrapper)
+from newrelic.api.html_insertion import insert_html_snippet
+from newrelic.api.transaction import current_transaction
+from newrelic.api.time_trace import record_exception
+from newrelic.api.transaction_name import wrap_transaction_name
+from newrelic.api.wsgi_application import WSGIApplicationWrapper
+
+from newrelic.common.object_wrapper import (FunctionWrapper, wrap_in_function,
+        wrap_post_function, wrap_function_wrapper, function_wrapper)
+from newrelic.common.object_names import callable_name
+from newrelic.config import extra_settings
+from newrelic.core.config import global_settings, ignore_status_code
 
 _logger = logging.getLogger(__name__)
 
 _boolean_states = {
-   '1': True, 'yes': True, 'true': True, 'on': True,
-   '0': False, 'no': False, 'false': False, 'off': False
+       '1': True, 'yes': True, 'true': True, 'on': True,
+       '0': False, 'no': False, 'false': False, 'off': False
 }
 
+
 def _setting_boolean(value):
     if value.lower() not in _boolean_states:
         raise ValueError('Not a boolean: %s' % value)
     return _boolean_states[value.lower()]
 
+
 def _setting_set(value):
     return set(value.split())
 
+
 _settings_types = {
     'browser_monitoring.auto_instrument': _setting_boolean,
-    'instrumentation.templates.inclusion_tag' : _setting_set,
+    'instrumentation.templates.inclusion_tag': _setting_set,
     'instrumentation.background_task.startup_timeout': float,
-    'instrumentation.scripts.django_admin' : _setting_set,
+    'instrumentation.scripts.django_admin': _setting_set,
 }
 
 _settings_defaults = {
     'browser_monitoring.auto_instrument': True,
     'instrumentation.templates.inclusion_tag': set(),
     'instrumentation.background_task.startup_timeout': 10.0,
-    'instrumentation.scripts.django_admin' : set(),
+    'instrumentation.scripts.django_admin': set(),
 }
 
 django_settings = extra_settings('import-hook:django',
         types=_settings_types, defaults=_settings_defaults)
 
-def should_ignore(exc, value, tb):
-    from django.http import Http404
 
-    if isinstance(value, Http404):
-        if ignore_status_code(404):
-            return True
-
-# Response middleware for automatically inserting RUM header and
-# footer into HTML response returned by application
-
-def browser_timing_middleware(request, response):
+def should_add_browser_timing(response, transaction):
 
     # Don't do anything if receive a streaming response which
     # was introduced in Django 1.5. Need to avoid this as there
@@ -69,29 +86,27 @@ def browser_timing_middleware(request, response):
     # deal with how to update the content length.
 
     if hasattr(response, 'streaming_content'):
-        return response
+        return False
 
     # Need to be running within a valid web transaction.
 
-    transaction = current_transaction()
-
     if not transaction:
-        return response
+        return False
 
     # Only insert RUM JavaScript headers and footers if enabled
     # in configuration and not already likely inserted.
 
     if not transaction.settings.browser_monitoring.enabled:
-        return response
+        return False
 
     if transaction.autorum_disabled:
-        return response
+        return False
 
     if not django_settings.browser_monitoring.auto_instrument:
-        return response
+        return False
 
     if transaction.rum_header_generated:
-        return response
+        return False
 
     # Only possible if the content type is one of the allowed
     # values. Normally this is just text/html, but optionally
@@ -102,19 +117,27 @@ def browser_timing_middleware(request, response):
     ctype = response.get('Content-Type', '').lower().split(';')[0]
 
     if ctype not in transaction.settings.browser_monitoring.content_type:
-        return response
+        return False
 
     # Don't risk it if content encoding already set.
 
     if response.has_header('Content-Encoding'):
-        return response
+        return False
 
     # Don't risk it if content is actually within an attachment.
 
     cdisposition = response.get('Content-Disposition', '').lower()
 
     if cdisposition.split(';')[0].strip().lower() == 'attachment':
-        return response
+        return False
+
+    return True
+
+
+# Response middleware for automatically inserting RUM header and
+# footer into HTML response returned by application
+
+def browser_timing_insertion(response, transaction):
 
     # No point continuing if header is empty. This can occur if
     # RUM is not enabled within the UI. It is assumed at this
@@ -150,33 +173,12 @@ def browser_timing_middleware(request, response):
                     len(result) - len(response.content))
 
         response.content = result
-        response['Content-Length'] = str(len(response.content))
+
+        if response.get('Content-Length', None):
+            response['Content-Length'] = str(len(response.content))
 
     return response
 
-def register_browser_timing_middleware(middleware):
-
-    # Inserts our middleware for inserting the RUM header and
-    # footer into the list of middleware. Must check for certain
-    # types of middleware which modify content as must always
-    # come before them. Otherwise is added last so that comes
-    # after any caching middleware. If don't do that then the
-    # inserted header and footer will end up being cached and
-    # then when served up from cache we would add a second
-    # header and footer, something we don't want.
-
-    content_type_modifying_middleware = [
-        'django.middleware.gzip:GZipMiddleware.process_response'
-    ]
-
-    for i in range(len(middleware)):
-        function = middleware[i]
-        name = callable_name(function)
-        if name in content_type_modifying_middleware:
-            middleware.insert(i, browser_timing_middleware)
-            break
-    else:
-        middleware.append(browser_timing_middleware)
 
 # Template tag functions for manually inserting RUM header and
 # footer into HTML response. A template tag library for
@@ -184,12 +186,18 @@ def register_browser_timing_middleware(middleware):
 # libraries when performing step to instrument the middleware.
 
 def newrelic_browser_timing_header():
+    from django.utils.safestring import mark_safe
+
     transaction = current_transaction()
-    return transaction and transaction.browser_timing_header() or ''
+    return transaction and mark_safe(transaction.browser_timing_header()) or ''
+
 
 def newrelic_browser_timing_footer():
+    from django.utils.safestring import mark_safe
+
     transaction = current_transaction()
-    return transaction and transaction.browser_timing_footer() or ''
+    return transaction and mark_safe(transaction.browser_timing_footer()) or ''
+
 
 # Addition of instrumentation for middleware. Can only do this
 # after Django itself has constructed the list of middleware. We
@@ -197,6 +205,7 @@ def newrelic_browser_timing_footer():
 
 middleware_instrumentation_lock = threading.Lock()
 
+
 def wrap_leading_middleware(middleware):
 
     # Wrapper to be applied to middleware executed prior to the
@@ -221,14 +230,14 @@ def wrap_leading_middleware(middleware):
 
             before = (transaction.name, transaction.group)
 
-            with FunctionTrace(transaction, name=name):
+            with FunctionTrace(name=name):
                 try:
                     return wrapped(*args, **kwargs)
 
                 finally:
                     # We want to name the transaction after this
                     # middleware but only if the transaction wasn't
-                    # named from within the middleware itself explicity.
+                    # named from within the middleware itself explicitly.
 
                     after = (transaction.name, transaction.group)
                     if before == after:
@@ -239,6 +248,7 @@ def wrap_leading_middleware(middleware):
     for wrapped in middleware:
         yield wrapper(wrapped)
 
+
 def wrap_view_middleware(middleware):
 
     # XXX This is no longer being used. The changes to strip the
@@ -286,14 +296,14 @@ def wrap_view_middleware(middleware):
 
             before = (transaction.name, transaction.group)
 
-            with FunctionTrace(transaction, name=name):
+            with FunctionTrace(name=name):
                 try:
                     return _wrapped(*args, **kwargs)
 
                 finally:
                     # We want to name the transaction after this
                     # middleware but only if the transaction wasn't
-                    # named from within the middleware itself explicity.
+                    # named from within the middleware itself explicitly.
 
                     after = (transaction.name, transaction.group)
                     if before == after:
@@ -304,6 +314,7 @@ def wrap_view_middleware(middleware):
     for wrapped in middleware:
         yield wrapper(wrapped)
 
+
 def wrap_trailing_middleware(middleware):
 
     # Wrapper to be applied to trailing middleware executed
@@ -320,12 +331,7 @@ def wrap_trailing_middleware(middleware):
         name = callable_name(wrapped)
 
         def wrapper(wrapped, instance, args, kwargs):
-            transaction = current_transaction()
-
-            if transaction is None:
-                return wrapped(*args, **kwargs)
-
-            with FunctionTrace(transaction, name=name):
+            with FunctionTrace(name=name):
                 return wrapped(*args, **kwargs)
 
         return FunctionWrapper(wrapped, wrapper)
@@ -333,6 +339,7 @@ def wrap_trailing_middleware(middleware):
     for wrapped in middleware:
         yield wrapper(wrapped)
 
+
 def insert_and_wrap_middleware(handler, *args, **kwargs):
 
     # Use lock to control access by single thread but also as
@@ -359,18 +366,8 @@ def insert_and_wrap_middleware(handler, *args, **kwargs):
     middleware_instrumentation_lock = None
 
     try:
-        # For response middleware, need to add in middleware for
-        # automatically inserting RUM header and footer. This is
-        # done first which means it gets wrapped and timed as
-        # well. Will therefore show in traces however that may
-        # be beneficial as highlights we are doing some magic
-        # and can see if it is taking too long on large
-        # responses.
 
-        if hasattr(handler, '_response_middleware'):
-            register_browser_timing_middleware(handler._response_middleware)
-
-        # Now wrap the middleware to undertake timing and name
+        # Wrap the middleware to undertake timing and name
         # the web transaction. The naming is done as lower
         # priority than that for view handler so view handler
         # name always takes precedence.
@@ -403,6 +400,53 @@ def insert_and_wrap_middleware(handler, *args, **kwargs):
     finally:
         lock.release()
 
+
+def _nr_wrapper_GZipMiddleware_process_response_(wrapped, instance, args,
+        kwargs):
+
+    transaction = current_transaction()
+
+    if transaction is None:
+        return wrapped(*args, **kwargs)
+
+    def _bind_params(request, response, *args, **kwargs):
+        return request, response
+
+    request, response = _bind_params(*args, **kwargs)
+
+    if should_add_browser_timing(response, transaction):
+        with FunctionTrace(
+                name=callable_name(browser_timing_insertion)):
+            response_with_browser = browser_timing_insertion(
+                    response, transaction)
+
+        return wrapped(request, response_with_browser)
+
+    return wrapped(request, response)
+
+
+def _bind_get_response(request, *args, **kwargs):
+    return request
+
+
+def _nr_wrapper_BaseHandler_get_response_(wrapped, instance, args, kwargs):
+    response = wrapped(*args, **kwargs)
+
+    transaction = current_transaction()
+
+    if transaction is None:
+        return response
+
+    request = _bind_get_response(*args, **kwargs)
+
+    if hasattr(request, '_nr_exc_info'):
+        if not ignore_status_code(response.status_code):
+            record_exception(*request._nr_exc_info)
+        delattr(request, '_nr_exc_info')
+
+    return response
+
+
 # Post import hooks for modules.
 
 def instrument_django_core_handlers_base(module):
@@ -414,6 +458,16 @@ def instrument_django_core_handlers_base(module):
     wrap_post_function(module, 'BaseHandler.load_middleware',
             insert_and_wrap_middleware)
 
+    wrap_function_wrapper(module, 'BaseHandler.get_response',
+            _nr_wrapper_BaseHandler_get_response_)
+
+
+def instrument_django_gzip_middleware(module):
+
+    wrap_function_wrapper(module, 'GZipMiddleware.process_response',
+            _nr_wrapper_GZipMiddleware_process_response_)
+
+
 def wrap_handle_uncaught_exception(middleware):
 
     # Wrapper to be applied to handler called when exceptions
@@ -433,20 +487,21 @@ def wrap_handle_uncaught_exception(middleware):
 
         def _wrapped(request, resolver, exc_info):
             transaction.set_transaction_name(name, priority=1)
-            transaction.record_exception(*exc_info)
+            record_exception(*exc_info)
 
             try:
                 return wrapped(request, resolver, exc_info)
 
             except:  # Catch all
-                transaction.record_exception(*sys.exc_info())
+                record_exception()
                 raise
 
-        with FunctionTrace(transaction, name=name):
+        with FunctionTrace(name=name):
             return _wrapped(*args, **kwargs)
 
     return FunctionWrapper(middleware, wrapper)
 
+
 def instrument_django_core_handlers_wsgi(module):
 
     # Wrap the WSGI application entry point. If this is also
@@ -466,11 +521,13 @@ def instrument_django_core_handlers_wsgi(module):
     # The handle_uncaught_exception() function produces a 500
     # error response page and otherwise suppresses the
     # exception, so last chance to do this as exception will not
-    # propogate up to the WSGI application.
+    # propagate up to the WSGI application.
+
+    if hasattr(module.WSGIHandler, 'handle_uncaught_exception'):
+        module.WSGIHandler.handle_uncaught_exception = (
+                wrap_handle_uncaught_exception(
+                module.WSGIHandler.handle_uncaught_exception))
 
-    module.WSGIHandler.handle_uncaught_exception = (
-            wrap_handle_uncaught_exception(
-            module.WSGIHandler.handle_uncaught_exception))
 
 def wrap_view_handler(wrapped, priority=3):
 
@@ -492,12 +549,18 @@ def wrap_view_handler(wrapped, priority=3):
 
         transaction.set_transaction_name(name, priority=priority)
 
-        with FunctionTrace(transaction, name=name):
+        with FunctionTrace(name=name):
             try:
                 return wrapped(*args, **kwargs)
 
             except:  # Catch all
-                transaction.record_exception(ignore_errors=should_ignore)
+                exc_info = sys.exc_info()
+                try:
+                    # Store exc_info on the request to check response code
+                    # prior to reporting
+                    args[0]._nr_exc_info = exc_info
+                except:
+                    record_exception(*exc_info)
                 raise
 
     result = FunctionWrapper(wrapped, wrapper)
@@ -505,6 +568,7 @@ def wrap_view_handler(wrapped, priority=3):
 
     return result
 
+
 def wrap_url_resolver(wrapped):
 
     # Wrap URL resolver. If resolver returns valid result then
@@ -535,10 +599,10 @@ def wrap_url_resolver(wrapped):
             # XXX This can raise a Resolver404. If this is not dealt
             # with, is this the source of our unnamed 404 requests.
 
-            with FunctionTrace(transaction, name=name, label=path):
+            with FunctionTrace(name=name, label=path):
                 result = wrapped(path)
 
-                if type(result) == type(()):
+                if type(result) is tuple:
                     callback, callback_args, callback_kwargs = result
                     result = (wrap_view_handler(callback, priority=5),
                             callback_args, callback_kwargs)
@@ -555,6 +619,7 @@ def wrap_url_resolver(wrapped):
 
     return FunctionWrapper(wrapped, wrapper)
 
+
 def wrap_url_resolver_nnn(wrapped, priority=1):
 
     # Wrapper to be applied to the URL resolver for errors.
@@ -567,13 +632,18 @@ def wrap_url_resolver_nnn(wrapped, priority=1):
         if transaction is None:
             return wrapped(*args, **kwargs)
 
-        with FunctionTrace(transaction, name=name):
-            callback, param_dict = wrapped(*args, **kwargs)
-            return (wrap_view_handler(callback, priority=priority),
-                    param_dict)
+        with FunctionTrace(name=name):
+            result = wrapped(*args, **kwargs)
+            if callable(result):
+                return wrap_view_handler(result, priority=priority)
+            else:
+                callback, param_dict = result
+                return (wrap_view_handler(callback, priority=priority),
+                        param_dict)
 
     return FunctionWrapper(wrapped, wrapper)
 
+
 def wrap_url_reverse(wrapped):
 
     # Wrap the URL resolver reverse lookup. Where the view
@@ -592,6 +662,7 @@ def wrap_url_reverse(wrapped):
 
     return FunctionWrapper(wrapped, wrapper)
 
+
 def instrument_django_core_urlresolvers(module):
 
     # Wrap method which maps a string version of a function
@@ -601,19 +672,22 @@ def instrument_django_core_urlresolvers(module):
     # and then reraises a ViewDoesNotExist exception with
     # details of the original error and traceback being
     # lost. We thus intercept it here so can capture that
-    # traceback which is otherwise lost. Although we ignore
-    # a Http404 exception here, it probably is never the
-    # case that one can be raised by get_callable().
+    # traceback which is otherwise lost.
 
-    wrap_error_trace(module, 'get_callable', ignore_errors=should_ignore)
+    wrap_error_trace(module, 'get_callable')
 
     # Wrap methods which resolves a request to a view handler.
     # This can be called against a resolver initialised against
     # a custom URL conf associated with a specific request, or a
     # resolver which uses the default URL conf.
 
-    module.RegexURLResolver.resolve = wrap_url_resolver(
-            module.RegexURLResolver.resolve)
+    if hasattr(module, 'RegexURLResolver'):
+        urlresolver = module.RegexURLResolver
+    else:
+        urlresolver = module.URLResolver
+
+    urlresolver.resolve = wrap_url_resolver(
+            urlresolver.resolve)
 
     # Wrap methods which resolve error handlers. For 403 and 404
     # we give these higher naming priority over any prior
@@ -623,20 +697,37 @@ def instrument_django_core_urlresolvers(module):
     # handler in place so error details identify the correct
     # transaction.
 
-    if hasattr(module.RegexURLResolver, 'resolve403'):
-        module.RegexURLResolver.resolve403 = wrap_url_resolver_nnn(
-                module.RegexURLResolver.resolve403, priority=3)
+    if hasattr(urlresolver, 'resolve403'):
+        urlresolver.resolve403 = wrap_url_resolver_nnn(
+                urlresolver.resolve403, priority=3)
 
-    module.RegexURLResolver.resolve404 = wrap_url_resolver_nnn(
-            module.RegexURLResolver.resolve404, priority=3)
+    if hasattr(urlresolver, 'resolve404'):
+        urlresolver.resolve404 = wrap_url_resolver_nnn(
+                urlresolver.resolve404, priority=3)
 
-    module.RegexURLResolver.resolve500 = wrap_url_resolver_nnn(
-            module.RegexURLResolver.resolve500, priority=1)
+    if hasattr(urlresolver, 'resolve500'):
+        urlresolver.resolve500 = wrap_url_resolver_nnn(
+                urlresolver.resolve500, priority=1)
+
+    if hasattr(urlresolver, 'resolve_error_handler'):
+        urlresolver.resolve_error_handler = wrap_url_resolver_nnn(
+                urlresolver.resolve_error_handler, priority=1)
 
     # Wrap function for performing reverse URL lookup to strip any
     # instrumentation wrapper when view handler is passed in.
 
-    module.reverse = wrap_url_reverse(module.reverse)
+    if hasattr(module, 'reverse'):
+        module.reverse = wrap_url_reverse(module.reverse)
+
+
+def instrument_django_urls_base(module):
+
+    # Wrap function for performing reverse URL lookup to strip any
+    # instrumentation wrapper when view handler is passed in.
+
+    if hasattr(module, 'reverse'):
+        module.reverse = wrap_url_reverse(module.reverse)
+
 
 def instrument_django_template(module):
 
@@ -660,6 +751,12 @@ def instrument_django_template(module):
         wrap_function_trace(module, 'Template.render',
                 name=template_name, group='Template/Render')
 
+    # Django 1.8 no longer has module.libraries. As automatic way is not
+    # preferred we can just skip this now.
+
+    if not hasattr(module, 'libraries'):
+        return
+
     # Register template tags used for manual insertion of RUM
     # header and footer.
     #
@@ -673,22 +770,16 @@ def instrument_django_template(module):
 
     module.libraries['django.templatetags.newrelic'] = library
 
+
 def wrap_template_block(wrapped):
-
-    name = callable_name(wrapped)
-
     def wrapper(wrapped, instance, args, kwargs):
-        transaction = current_transaction()
-
-        if transaction is None:
-            return wrapped(*args, **kwargs)
-
-        with FunctionTrace(transaction, name=instance.name,
+        with FunctionTrace(name=instance.name,
                 group='Template/Block'):
             return wrapped(*args, **kwargs)
 
     return FunctionWrapper(wrapped, wrapper)
 
+
 def instrument_django_template_loader_tags(module):
 
     # Wrap template block node for timing, naming the node after
@@ -697,6 +788,7 @@ def instrument_django_template_loader_tags(module):
 
     module.BlockNode.render = wrap_template_block(module.BlockNode.render)
 
+
 def instrument_django_core_servers_basehttp(module):
 
     # Allow 'runserver' to be used with Django <= 1.3. To do
@@ -723,18 +815,9 @@ def instrument_django_core_servers_basehttp(module):
     # instead which will likely need to be dealt with via
     # instrumentation of the wsgiref module or some other means.
 
-    import django
-
-    framework = ('Django', django.get_version())
-
     def wrap_wsgi_application_entry_point(server, application, **kwargs):
-      return ((server, WSGIApplicationWrapper(application,
-              framework='Django'),), kwargs)
-
-    # XXX Because of risk of people still trying to use the
-    # inbuilt Django development server and since the code is
-    # not going to be changed, could just patch it to fix
-    # problem and the instrumentation we need.
+        return ((server, WSGIApplicationWrapper(application,
+            framework='Django'),), kwargs)
 
     if (not hasattr(module, 'simple_server') and
             hasattr(module.ServerHandler, 'run')):
@@ -751,15 +834,14 @@ def instrument_django_core_servers_basehttp(module):
             finally:
                 self.close()
 
-
         def close(self):
             if self.result is not None:
                 try:
                     self.request_handler.log_request(
-                            self.status.split(' ',1)[0], self.bytes_sent)
+                            self.status.split(' ', 1)[0], self.bytes_sent)
                 finally:
                     try:
-                        if hasattr(self.result,'close'):
+                        if hasattr(self.result, 'close'):
                             self.result.close()
                     finally:
                         self.result = None
@@ -772,21 +854,24 @@ def instrument_django_core_servers_basehttp(module):
         # Leaving this out for now to see whether weakref solves
         # the problem.
 
-        #module.ServerHandler.run = run
-        #module.ServerHandler.close = close
+        # module.ServerHandler.run = run
+        # module.ServerHandler.close = close
 
         # Now wrap it with our instrumentation.
 
         wrap_in_function(module, 'ServerHandler.run',
                 wrap_wsgi_application_entry_point)
 
+
 def instrument_django_contrib_staticfiles_views(module):
     if not hasattr(module.serve, '_nr_django_view_handler'):
         module.serve = wrap_view_handler(module.serve, priority=3)
 
+
 def instrument_django_contrib_staticfiles_handlers(module):
     wrap_transaction_name(module, 'StaticFilesHandler.serve')
 
+
 def instrument_django_views_debug(module):
 
     # Wrap methods for handling errors when Django debug
@@ -802,6 +887,7 @@ def instrument_django_views_debug(module):
     module.technical_500_response = wrap_view_handler(
             module.technical_500_response, priority=1)
 
+
 def wrap_view_dispatch(wrapped):
 
     # Wrapper to be applied to dispatcher for class based views.
@@ -829,33 +915,62 @@ def wrap_view_dispatch(wrapped):
             handler = view.http_method_not_allowed
 
         name = callable_name(handler)
-        transaction.set_transaction_name(name, priority=4)
 
-        with FunctionTrace(transaction, name=name):
+        # The priority to be used when naming the transaction is
+        # bit tricky. If the transaction name is already that of
+        # the class based view, but not the method, then we want
+        # the name of the method to override. This can occur
+        # where the class based view was registered directly in
+        # urls.py as the view handler. In this case we use the
+        # priority of 5, matching what would be used by the view
+        # handler so that it can override the transaction name.
+        #
+        # If however the transaction name is unrelated, we
+        # preferably don't want it overridden. This can happen
+        # where the class based view was invoked explicitly
+        # within an existing view handler. In this case we use
+        # the priority of 4 so it will not override the view
+        # handler name where used as the transaction name.
+
+        priority = 4
+
+        if transaction.group == 'Function':
+            if transaction.name == callable_name(view):
+                priority = 5
+
+        transaction.set_transaction_name(name, priority=priority)
+
+        with FunctionTrace(name=name):
             return wrapped(*args, **kwargs)
 
     return FunctionWrapper(wrapped, wrapper)
 
+
 def instrument_django_views_generic_base(module):
     module.View.dispatch = wrap_view_dispatch(module.View.dispatch)
 
+
 def instrument_django_http_multipartparser(module):
     wrap_function_trace(module, 'MultiPartParser.parse')
 
+
 def instrument_django_core_mail(module):
     wrap_function_trace(module, 'mail_admins')
     wrap_function_trace(module, 'mail_managers')
     wrap_function_trace(module, 'send_mail')
 
+
 def instrument_django_core_mail_message(module):
     wrap_function_trace(module, 'EmailMessage.send')
 
+
 def _nr_wrapper_BaseCommand___init___(wrapped, instance, args, kwargs):
     instance.handle = FunctionTraceWrapper(instance.handle)
     if hasattr(instance, 'handle_noargs'):
         instance.handle_noargs = FunctionTraceWrapper(instance.handle_noargs)
     return wrapped(*args, **kwargs)
 
+
 def _nr_wrapper_BaseCommand_run_from_argv_(wrapped, instance, args, kwargs):
     def _args(argv, *args, **kwargs):
         return argv
@@ -876,21 +991,18 @@ def _nr_wrapper_BaseCommand_run_from_argv_(wrapped, instance, args, kwargs):
     with BackgroundTask(application, subcommand, 'Django'):
         return wrapped(*args, **kwargs)
 
+
 def instrument_django_core_management_base(module):
     wrap_function_wrapper(module, 'BaseCommand.__init__',
             _nr_wrapper_BaseCommand___init___)
     wrap_function_wrapper(module, 'BaseCommand.run_from_argv',
             _nr_wrapper_BaseCommand_run_from_argv_)
 
+
 @function_wrapper
 def _nr_wrapper_django_inclusion_tag_wrapper_(wrapped, instance,
         args, kwargs):
 
-    transaction = current_transaction()
-
-    if transaction is None:
-        return wrapped(*args, **kwargs)
-
     name = hasattr(wrapped, '__name__') and wrapped.__name__
 
     if name is None:
@@ -903,9 +1015,10 @@ def _nr_wrapper_django_inclusion_tag_wrapper_(wrapped, instance,
     if '*' not in tags and name not in tags and qualname not in tags:
         return wrapped(*args, **kwargs)
 
-    with FunctionTrace(transaction, name, group='Template/Tag'):
+    with FunctionTrace(name, group='Template/Tag'):
         return wrapped(*args, **kwargs)
 
+
 @function_wrapper
 def _nr_wrapper_django_inclusion_tag_decorator_(wrapped, instance,
         args, kwargs):
@@ -919,21 +1032,18 @@ def _nr_wrapper_django_inclusion_tag_decorator_(wrapped, instance,
 
     return wrapped(func, *_args, **_kwargs)
 
+
 def _nr_wrapper_django_template_base_Library_inclusion_tag_(wrapped,
         instance, args, kwargs):
 
     return _nr_wrapper_django_inclusion_tag_decorator_(
             wrapped(*args, **kwargs))
 
+
 @function_wrapper
 def _nr_wrapper_django_template_base_InclusionNode_render_(wrapped,
         instance, args, kwargs):
 
-    transaction = current_transaction()
-
-    if transaction is None:
-        return wrapped(*args, **kwargs)
-
     if wrapped.__self__ is None:
         return wrapped(*args, **kwargs)
 
@@ -944,9 +1054,10 @@ def _nr_wrapper_django_template_base_InclusionNode_render_(wrapped,
 
     name = wrapped.__self__._nr_file_name
 
-    with FunctionTrace(transaction, name, 'Template/Include'):
+    with FunctionTrace(name, 'Template/Include'):
         return wrapped(*args, **kwargs)
 
+
 def _nr_wrapper_django_template_base_generic_tag_compiler_(wrapped, instance,
         args, kwargs):
 
@@ -976,6 +1087,7 @@ def _nr_wrapper_django_template_base_generic_tag_compiler_(wrapped, instance,
 
     return wrapped(*args, **kwargs)
 
+
 def _nr_wrapper_django_template_base_Library_tag_(wrapped, instance,
         args, kwargs):
 
@@ -999,9 +1111,9 @@ def _nr_wrapper_django_template_base_Library_tag_(wrapped, instance,
         # Django < 1.4 uses their home-grown "curry" function,
         # not functools.partial.
 
-        if (hasattr(compile_function, 'func_closure')
-                and hasattr(compile_function, '__name__')
-                and compile_function.__name__ == '_curried'):
+        if (hasattr(compile_function, 'func_closure') and
+                hasattr(compile_function, '__name__') and
+                compile_function.__name__ == '_curried'):
 
             # compile_function here is generic_tag_compiler(), which has been
             # curried. To get node_class, we first get the function obj, args,
@@ -1035,8 +1147,8 @@ def _nr_wrapper_django_template_base_Library_tag_(wrapped, instance,
     for i in range(1, stack_levels + 1):
         frame = sys._getframe(i)
 
-        if ('generic_tag_compiler' in frame.f_code.co_names
-                and 'file_name' in frame.f_code.co_freevars):
+        if ('generic_tag_compiler' in frame.f_code.co_names and
+                'file_name' in frame.f_code.co_freevars):
             file_name = frame.f_locals.get('file_name')
 
     if file_name is None:
@@ -1049,6 +1161,7 @@ def _nr_wrapper_django_template_base_Library_tag_(wrapped, instance,
 
     return wrapped(*args, **kwargs)
 
+
 def instrument_django_template_base(module):
     global module_django_template_base
     module_django_template_base = module
@@ -1056,11 +1169,56 @@ def instrument_django_template_base(module):
     settings = global_settings()
 
     if 'django.instrumentation.inclusion-tags.r1' in settings.feature_flag:
-        wrap_function_wrapper(module, 'generic_tag_compiler',
-                _nr_wrapper_django_template_base_generic_tag_compiler_)
 
-        wrap_function_wrapper(module, 'Library.tag',
-                _nr_wrapper_django_template_base_Library_tag_)
+        if hasattr(module, 'generic_tag_compiler'):
+            wrap_function_wrapper(module, 'generic_tag_compiler',
+                    _nr_wrapper_django_template_base_generic_tag_compiler_)
 
-        wrap_function_wrapper(module, 'Library.inclusion_tag',
+        if hasattr(module, 'Library'):
+            wrap_function_wrapper(module, 'Library.tag',
+                    _nr_wrapper_django_template_base_Library_tag_)
+
+            wrap_function_wrapper(module, 'Library.inclusion_tag',
                 _nr_wrapper_django_template_base_Library_inclusion_tag_)
+
+
+def _nr_wrap_converted_middleware_(middleware, name):
+
+    @function_wrapper
+    def _wrapper(wrapped, instance, args, kwargs):
+        transaction = current_transaction()
+
+        if transaction is None:
+            return wrapped(*args, **kwargs)
+
+        transaction.set_transaction_name(name, priority=2)
+
+        with FunctionTrace(name=name):
+            return wrapped(*args, **kwargs)
+
+    return _wrapper(middleware)
+
+
+def _nr_wrapper_convert_exception_to_response_(wrapped, instance, args,
+        kwargs):
+
+    def _bind_params(original_middleware, *args, **kwargs):
+        return original_middleware
+
+    original_middleware = _bind_params(*args, **kwargs)
+    converted_middleware = wrapped(*args, **kwargs)
+    name = callable_name(original_middleware)
+
+    return _nr_wrap_converted_middleware_(converted_middleware, name)
+
+
+def instrument_django_core_handlers_exception(module):
+
+    if hasattr(module, 'convert_exception_to_response'):
+        wrap_function_wrapper(module, 'convert_exception_to_response',
+                _nr_wrapper_convert_exception_to_response_)
+
+    if hasattr(module, 'handle_uncaught_exception'):
+        module.handle_uncaught_exception = (
+                wrap_handle_uncaught_exception(
+                module.handle_uncaught_exception))
