@@ -1,13 +1,29 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import sys
 import types
 
 import newrelic.api.transaction
-import newrelic.api.name_transaction
+import newrelic.api.transaction_name
 import newrelic.api.function_trace
 import newrelic.api.error_trace
 import newrelic.api.object_wrapper
 import newrelic.api.import_hook
 
+from newrelic.api.time_trace import record_exception
+
 def name_controller(self, environ, start_response):
     action = environ['pylons.routes_dict']['action']
     return "%s.%s" % (newrelic.api.object_wrapper.callable_name(self), action)
@@ -36,7 +52,7 @@ class capture_error(object):
             except webob_exc.HTTPException:
                 raise
             except:  # Catch all
-                current_transaction.record_exception(*sys.exc_info())
+                record_exception()
                 raise
         else:
             return self.__wrapped(*args, **kwargs)
@@ -50,7 +66,7 @@ def instrument(module):
         newrelic.api.error_trace.wrap_error_trace(module, 'PylonsApp.__call__')
 
     elif module.__name__ == 'pylons.controllers.core':
-        newrelic.api.name_transaction.wrap_name_transaction(
+        newrelic.api.transaction_name.wrap_transaction_name(
                 module, 'WSGIController.__call__', name_controller)
         newrelic.api.function_trace.wrap_function_trace(
                 module, 'WSGIController.__call__')
