@@ -1,3 +1,17 @@
+# Copyright 2010 New Relic, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import logging
 import sys
 import weakref
@@ -10,6 +24,8 @@ import newrelic.api.web_transaction
 import newrelic.api.function_trace
 import newrelic.api.error_trace
 
+from newrelic.api.time_trace import record_exception
+
 _logger = logging.getLogger(__name__)
 
 class RequestProcessWrapper(object):
@@ -61,7 +77,7 @@ class RequestProcessWrapper(object):
 
         # Now start recording the actual web transaction.
 
-        transaction = newrelic.api.web_transaction.WebTransaction(
+        transaction = newrelic.api.web_transaction.WSGIWebTransaction(
                 application, environ)
 
         if not transaction.enabled:
@@ -93,7 +109,7 @@ class RequestProcessWrapper(object):
             # it is not done yet and register deferred callbacks to
             # complete the request.
 
-            with newrelic.api.function_trace.FunctionTrace(transaction,
+            with newrelic.api.function_trace.FunctionTrace(
                     name='Request/Process', group='Python/Twisted'):
                 result = self._nr_next_object()
 
@@ -114,7 +130,7 @@ class RequestProcessWrapper(object):
             else:
                 self._nr_instance._nr_wait_function_trace = \
                         newrelic.api.function_trace.FunctionTrace(
-                        transaction, name='Deferred/Wait',
+                        name='Deferred/Wait',
                         group='Python/Twisted')
 
                 self._nr_instance._nr_wait_function_trace.__enter__()
@@ -207,12 +223,12 @@ class RequestFinishWrapper(object):
             # Most likely the finish() call would never fail anyway.
 
             try:
-                with newrelic.api.function_trace.FunctionTrace(transaction,
+                with newrelic.api.function_trace.FunctionTrace(
                         name='Request/Finish', group='Python/Twisted'):
                     result = self._nr_next_object()
 
             except:  # Catch all
-                transaction.record_exception(*sys.exc_info())
+                record_exception(*sys.exc_info())
                 raise
 
         elif self._nr_instance._nr_wait_function_trace:
@@ -231,7 +247,7 @@ class RequestFinishWrapper(object):
                 self._nr_instance._nr_wait_function_trace.__exit__(
                         None, None, None)
 
-                with newrelic.api.function_trace.FunctionTrace(transaction,
+                with newrelic.api.function_trace.FunctionTrace(
                         name='Request/Finish', group='Python/Twisted'):
                     result = self._nr_next_object()
 
@@ -250,7 +266,7 @@ class RequestFinishWrapper(object):
             # This should be the case where finish() is being called in
             # the original render() function.
 
-            with newrelic.api.function_trace.FunctionTrace(transaction,
+            with newrelic.api.function_trace.FunctionTrace(
                     name='Request/Finish', group='Python/Twisted'):
                 result = self._nr_next_object()
 
@@ -307,9 +323,9 @@ class ResourceRenderWrapper(object):
         name = "%s.render_%s" % (
                 newrelic.api.object_wrapper.callable_name(
                 instance), request.method)
-        transaction.name_transaction(name, priority=1)
+        transaction.set_transaction_name(name, priority=1)
 
-        with newrelic.api.function_trace.FunctionTrace(transaction, name):
+        with newrelic.api.function_trace.FunctionTrace(name):
             return self._nr_next_object(*args)
 
 class DeferredUserList(UserList.UserList):
@@ -452,9 +468,9 @@ class DeferredCallbacksWrapper(object):
             # Call the deferred and capture any errors that may come
             # back from it.
 
-            with newrelic.api.error_trace.ErrorTrace(transaction):
+            with newrelic.api.error_trace.ErrorTrace():
                 with newrelic.api.function_trace.FunctionTrace(
-                        transaction, name='Deferred/Call',
+                        name='Deferred/Call',
                         group='Python/Twisted'):
                     return self._nr_next_object()
 
@@ -476,7 +492,7 @@ class DeferredCallbacksWrapper(object):
 
                 request._nr_wait_function_trace = \
                         newrelic.api.function_trace.FunctionTrace(
-                        transaction, name='Deferred/Wait',
+                        name='Deferred/Wait',
                         group='Python/Twisted')
 
                 request._nr_wait_function_trace.__enter__()
@@ -494,9 +510,8 @@ class InlineGeneratorWrapper(object):
         name = newrelic.api.object_wrapper.callable_name(self._nr_wrapped)
         iterable = iter(self._nr_generator)
         while 1:
-            transaction = newrelic.api.transaction.current_transaction()
             with newrelic.api.function_trace.FunctionTrace(
-                  transaction, name, group='Python/Twisted/Generator'):
+                  name, group='Python/Twisted/Generator'):
                 yield next(iterable)
 
 class InlineCallbacksWrapper(object):
